@@ -26,7 +26,7 @@
 /* #undef LLVM_DATADIR */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "mips64-unknown-openbsd5.5"
+#define LLVM_DEFAULT_TARGET_TRIPLE "amd64-unknown-openbsd5.9"
 
 /* Installation directory for documentation */
 /* #undef LLVM_DOCSDIR */
@@ -38,10 +38,10 @@
 /* #undef LLVM_ETCDIR */
 
 /* Has gcc/MSVC atomic intrinsics */
-#define LLVM_HAS_ATOMICS 0
+#define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "mips64-unknown-openbsd5.5"
+#define LLVM_HOST_TRIPLE "amd64-unknown-openbsd5.9"
 
 /* Installation directory for include files */
 /* #undef LLVM_INCLUDEDIR */
@@ -49,32 +49,29 @@
 /* Installation directory for .info files */
 /* #undef LLVM_INFODIR */
 
-/* Installation directory for libraries */
-/* #undef LLVM_LIBDIR */
-
 /* Installation directory for man pages */
 /* #undef LLVM_MANDIR */
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH Mips
+#define LLVM_NATIVE_ARCH X86
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeMipsAsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeX86AsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeMipsAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeX86AsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeMipsDisassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeX86Disassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeMipsTarget
+#define LLVM_NATIVE_TARGET LLVMInitializeX86Target
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeMipsTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeX86TargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeMipsTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeX86TargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -122,9 +119,6 @@
 #define LLVM_VERSION_MAJOR 3
 
 /* Minor version of the LLVM API */
-#define LLVM_VERSION_MINOR 3
-
-/* Define to 1 if you have the <sanitizer/msan_interface.h> header file. */
-/* #undef HAVE_SANITIZER_MSAN_INTERFACE_H */
+#define LLVM_VERSION_MINOR 5
 
 #endif
